package reason-standard

  1. Overview
  2. Docs

Module Standard.Container

Interfaces for use with container types like Array or List

module type Sum = sig ... end

Modules which conform to this signature can be used with functions like Array.sum or List.sum