package reason-standard

  1. Overview
  2. Docs

Source file Core.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
674
675
676
677
678
679
680
681
682
683
684
685
686
687
688
689
690
691
692
693
694
695
696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
745
746
747
748
749
750
751
752
753
754
755
756
757
758
759
760
761
762
763
764
765
766
767
768
769
770
771
772
773
774
775
776
777
778
779
780
781
782
783
784
785
786
787
788
789
790
791
792
793
794
795
796
797
798
799
800
801
802
803
804
805
806
807
808
809
810
811
812
813
814
815
816
817
818
819
820
821
822
823
824
825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
843
844
845
846
847
848
849
850
851
852
853
854
855
856
857
858
859
860
861
862
863
864
865
866
867
868
869
870
871
872
873
874
875
876
877
878
879
880
881
882
883
884
885
886
887
888
889
890
891
892
893
894
895
896
897
898
899
900
901
902
903
904
905
906
907
908
909
910
911
912
913
914
915
916
917
918
919
920
921
922
923
924
925
926
927
928
929
930
931
932
933
934
935
936
937
938
939
940
941
942
943
944
945
946
947
948
949
950
951
952
953
954
955
956
957
958
959
960
961
962
963
964
965
966
967
968
969
970
971
972
973
974
975
976
977
978
979
980
981
982
983
984
985
986
987
988
989
990
991
992
993
994
995
996
997
998
999
1000
1001
1002
1003
1004
1005
1006
1007
1008
1009
1010
1011
1012
1013
1014
1015
1016
1017
1018
1019
1020
1021
1022
1023
1024
1025
1026
1027
1028
1029
1030
1031
1032
1033
1034
1035
1036
1037
1038
1039
1040
1041
1042
1043
1044
1045
1046
1047
1048
1049
1050
1051
1052
1053
1054
1055
1056
1057
1058
1059
1060
1061
1062
1063
1064
1065
1066
1067
1068
1069
1070
1071
1072
1073
1074
1075
1076
1077
1078
1079
1080
1081
1082
1083
1084
1085
1086
1087
1088
1089
1090
1091
1092
1093
1094
1095
1096
1097
1098
1099
1100
1101
1102
1103
1104
1105
1106
1107
1108
1109
1110
1111
1112
1113
1114
1115
1116
1117
1118
1119
1120
1121
1122
1123
1124
1125
1126
1127
1128
1129
1130
1131
1132
1133
1134
1135
1136
1137
1138
1139
1140
1141
1142
1143
1144
1145
1146
1147
1148
1149
1150
1151
1152
1153
1154
1155
1156
1157
1158
1159
1160
1161
1162
1163
1164
1165
1166
1167
1168
1169
1170
1171
1172
1173
1174
1175
1176
1177
1178
1179
1180
1181
1182
1183
1184
1185
1186
1187
1188
1189
1190
1191
1192
1193
1194
1195
1196
1197
1198
1199
1200
1201
1202
1203
1204
1205
1206
1207
1208
1209
1210
1211
1212
1213
1214
1215
1216
1217
1218
1219
1220
1221
1222
1223
1224
1225
1226
1227
1228
1229
1230
1231
1232
1233
1234
1235
1236
1237
1238
1239
1240
1241
1242
1243
1244
1245
1246
1247
1248
1249
1250
1251
1252
1253
1254
1255
1256
1257
1258
1259
1260
1261
1262
1263
1264
1265
1266
1267
1268
1269
1270
1271
1272
1273
1274
1275
1276
1277
1278
1279
1280
1281
1282
1283
1284
1285
1286
1287
1288
1289
1290
1291
1292
1293
1294
1295
1296
1297
1298
1299
1300
1301
1302
1303
1304
1305
1306
1307
1308
1309
1310
1311
1312
1313
1314
1315
1316
1317
1318
1319
1320
1321
1322
1323
1324
1325
1326
1327
1328
1329
1330
1331
1332
1333
1334
1335
1336
1337
1338
1339
1340
1341
1342
1343
1344
1345
1346
1347
1348
1349
1350
1351
1352
1353
1354
1355
1356
1357
1358
1359
1360
1361
1362
1363
1364
1365
1366
1367
1368
1369
1370
1371
1372
1373
1374
1375
1376
1377
1378
1379
1380
1381
1382
1383
1384
1385
1386
1387
1388
1389
1390
1391
1392
1393
1394
1395
1396
1397
1398
1399
1400
1401
1402
1403
1404
1405
1406
1407
1408
1409
1410
1411
1412
1413
1414
1415
1416
1417
1418
1419
1420
1421
1422
1423
1424
1425
1426
1427
1428
1429
1430
1431
1432
1433
1434
1435
1436
1437
1438
1439
1440
1441
1442
1443
1444
1445
1446
1447
1448
1449
1450
1451
1452
1453
1454
1455
1456
1457
1458
1459
1460
1461
1462
1463
1464
1465
1466
1467
1468
1469
1470
1471
1472
1473
1474
1475
1476
1477
1478
1479
1480
1481
1482
1483
1484
1485
1486
1487
1488
1489
1490
1491
1492
1493
1494
1495
1496
1497
1498
1499
1500
1501
1502
1503
1504
1505
1506
1507
1508
1509
1510
1511
1512
1513
1514
1515
1516
1517
1518
1519
1520
1521
1522
1523
1524
1525
1526
1527
1528
1529
1530
1531
1532
1533
1534
1535
1536
1537
1538
1539
1540
1541
1542
1543
1544
1545
1546
1547
1548
1549
1550
1551
1552
1553
1554
1555
1556
1557
1558
1559
1560
1561
1562
1563
1564
1565
1566
1567
1568
1569
1570
1571
1572
1573
1574
1575
1576
1577
1578
1579
1580
1581
1582
1583
1584
1585
1586
1587
1588
1589
1590
1591
1592
1593
1594
1595
1596
1597
1598
1599
1600
1601
1602
1603
1604
1605
1606
1607
module Comparator = struct
  type ('a, 'identity) t = ('a, 'identity) Base.Comparator.t

  type ('a, 'identity) comparator = ('a, 'identity) t


  module type T = sig 
    type nonrec t    
    val compare : t -> t -> int
  end

  module type S = sig
    type nonrec t
    type identity
    val comparator : (t, identity) comparator
  end 

  type ('a, 'identity) s = (module S with type identity = 'identity and type t = 'a)

  (* let ofBaseComparator (baseComparator : ('a, 'id) Base.Map.comparator) : ('a, 'id) s = Obj.magic baseComparator *)

  let toBaseComparator (comparator : ('a, 'id) s) : ('a, 'id) Base.Map.comparator = Obj.magic comparator

  let opaque _ = Base.Sexp.Atom "<opaque>"

  let make ~compare =
    Obj.magic (Base.Comparator.make ~compare ~sexp_of_t:opaque)    

  module Make (M : T) = struct
    module BaseComparator = Base.Comparator.Make(struct
      include M
      let compare = M.compare
      let sexp_of_t = opaque
    end)
    include BaseComparator
    type identity = BaseComparator.comparator_witness
    let comparator = BaseComparator.comparator
  end  
end

module Bool = struct
  type t = bool

  external ( && ) : bool -> bool -> bool = "%sequand"

  external ( || ) : bool -> bool -> bool = "%sequor"

  let xor a b = (a && not b) || ((not a) && b)

  let not = not

  let negate f t = not (f t)

  let equal = ( = )

  let compare = compare

  let ofInt i = match i with 0 -> Some false | 1 -> Some true | _ -> None

  let ofString string =
    match string with "false" -> Some false | "true" -> Some true | _ -> None

  let toString = function true -> "true" | false -> "false"

  let toInt t = match t with true -> 1 | false -> 0
end

module Char = struct
  type t = char

  let toCode (c : char) = (Base.Char.to_int c : int)

  let ofCode (i : int) =
    (if 0 <= i && i <= 255 then Some (Char.chr i) else None : char option)

  let toString = Base.Char.to_string

  let ofString (str : string) =
    (match String.length str with 1 -> Some str.[0] | _ -> None : char option)

  let toDigit char =
    match char with '0' .. '9' -> Some (toCode char - toCode '0') | _ -> None

  let toLowercase = Base.Char.lowercase

  let toUppercase = Base.Char.uppercase

  let isLowercase = Base.Char.is_lowercase

  let isUppercase = Base.Char.is_uppercase

  let isLetter = Base.Char.is_alpha

  let isDigit = Base.Char.is_digit

  let isAlphanumeric = Base.Char.is_alphanum

  let isPrintable = Base.Char.is_print

  let isWhitespace = Base.Char.is_whitespace

  let equal: char -> char -> bool = ( = )

  let compare = compare

  type identity = Base.Char.comparator_witness

  let comparator = Base.Char.comparator
end

module Fun = struct
  external identity : 'a -> 'a = "%identity"

  external ignore : _ -> unit = "%ignore"

  let constant a _ = a

  let sequence a b = ignore a ; b

  let flip f a b = f b a

  let apply f a = f a

  let ( <| ) f a = f a

  external pipe : 'a -> ('a -> 'b) -> 'b = "%revapply"

  external ( |> ) : 'a -> ('a -> 'b) -> 'b = "%revapply"

  let compose f g a = f (g a)

  let ( << ) = compose

  let composeRight f g a = g (f a)

  let ( >> ) = composeRight

  let tap value ~f = f value ; value

  let rec times n ~f =
    if n <= 0 then ()
    else (
      f () ;
      times (n - 1) ~f )

  let forever f =
    try
      while true do
        f ()
      done;
      failwith "[while true] managed to return, you are in trouble now."
    with exn -> exn

  let curry (f : 'a * 'b -> 'c) (a : 'a) (b : 'b) = (f (a, b) : 'c)

  let uncurry (f : 'a -> 'b -> 'c) ((a, b) : 'a * 'b) = (f a b : 'c)

  let curry3 (f : 'a * 'b * 'c -> 'd) (a : 'a) (b : 'b) (c : 'c) =
    (f (a, b, c) : 'd)

  let uncurry3 (f : 'a -> 'b -> 'c -> 'd) ((a, b, c) : 'a * 'b * 'c) =
    (f a b c : 'd)
end

module Container = struct
  module type Sum = sig
    type t

    val zero : t

    val add : t -> t -> t
  end
end

module Tuple = struct
  type ('a, 'b) t = 'a * 'b

  let make a b = (a, b)

  let ofArray = function [|a; b|] -> Some (a, b) | _ -> None

  let ofList = function a :: b :: _ -> Some (a, b) | _ -> None

  let first (a, _) = a

  let second (_, b) = b

  let mapFirst (a, b) ~f = (f a, b)

  let mapSecond (a, b) ~f = (a, f b)

  let mapEach (a, b) ~f ~g = (f a, g b)

  let mapAll (a, b) ~f = (f a, f b)

  let swap (a, b) = (b, a)

  let toArray (a, b) = [|a; b|]

  let toList (a, b) = [a; b]

  let equal equalFirst equalSecond (a, b) (a', b') =
    equalFirst a a' && equalSecond b b'

  let compare compareFirst compareSecond (a, b) (a', b') =
    match compareFirst a a' with 0 -> compareSecond b b' | result -> result
end

module Tuple3 = struct
  type ('a, 'b, 'c) t = 'a * 'b * 'c

  let make a b c = (a, b, c)

  let ofArray = function [|a; b; c|] -> Some (a, b, c) | _ -> None

  let ofList = function a :: b :: c :: _ -> Some (a, b, c) | _ -> None

  let first ((a, _, _) : 'a * 'b * 'c) = (a : 'a)

  let second ((_, b, _) : 'a * 'b * 'c) = (b : 'b)

  let third ((_, _, c) : 'a * 'b * 'c) = (c : 'c)

  let initial ((a, b, _) : 'a * 'b * 'c) = ((a, b) : 'a * 'b)

  let tail ((_, b, c) : 'a * 'b * 'c) = ((b, c) : 'b * 'c)

  let mapFirst (a, b, c) ~f = (f a, b, c)

  let mapSecond (a, b, c) ~f = (a, f b, c)

  let mapThird (a, b, c) ~f = (a, b, f c)

  let mapEach (a, b, c) ~f ~g ~h = (f a, g b, h c)

  let mapAll (a1, a2, a3) ~f = (f a1, f a2, f a3)

  let rotateLeft ((a, b, c) : 'a * 'b * 'c) = ((b, c, a) : 'b * 'c * 'a)

  let rotateRight ((a, b, c) : 'a * 'b * 'c) = ((c, a, b) : 'c * 'a * 'b)

  let toArray (a, b, c) = [|a; b; c|]

  let toList ((a, b, c) : 'a * 'a * 'a) = ([a; b; c] : 'a list)

  let equal equalFirst equalSecond equalThird (a, b, c) (a', b', c') =
    equalFirst a a' && equalSecond b b' && equalThird c c'

  let compare compareFirst compareSecond compareThird (a, b, c) (a', b', c') =
    match compareFirst a a' with
    | 0 -> (
      match compareSecond b b' with 0 -> compareThird c c' | result -> result )
    | result ->
        result
end

module Option = struct
  type 'a t = 'a option

  let some a = Some a

  let isSome = Option.is_some

  let isNone = Option.is_none

  let and_ ta tb = match isSome ta with true -> tb | false -> None

  let or_ ta tb = match isSome ta with true -> ta | false -> tb

  let flatMap t ~f = match t with Some x -> f x | None -> None

  let flatten = Option.join

  let both a b =
    match (a, b) with (Some a, Some b) -> Some (a, b) | _ -> None

  let map t ~f = Option.map f t

  let map2 (ta : 'a t) (tb : 'b t) ~(f : 'a -> 'b -> 'c) =
    (match (ta, tb) with (Some a, Some b) -> Some (f a b) | _ -> None : 'c t)

  let get t ~default = match t with None -> default | Some value -> value

  let ( |? ) t default = get t ~default

  let getUnsafe x =
    match x with
    | None ->
        raise (Invalid_argument "Option.getUnsafe called with None")
    | Some x ->
        x

  let forEach t ~f = Option.iter f t

  let toArray t = match t with None -> [||] | Some value -> [|value|]

  let toList t = match t with None -> [] | Some value -> [value]

  module Infix = struct    
    let ( >>= ) t f = flatMap t ~f

    let ( >>| ) t f = map t ~f
  end

  let equal equal a b =
    match (a, b) with
    | (None, None) ->
        true
    | (Some a', Some b') ->
        equal a' b'
    | _ ->
        false

  let compare compare a b =
    match (a, b) with
    | (None, None) ->
        0
    | (Some a', Some b') ->
        compare a' b'
    | (None, Some _) ->
        -1
    | (Some _, None) ->
        1
end

module Result = struct
  type ('ok, 'error) t = ('ok, 'error) Result.t

  let ok = Result.ok

  let error = Result.error

  let ofOption ma ~error =
    match ma with None -> Result.Error error | Some right -> Result.Ok right

  let isOk = Result.is_ok

  let isError = Result.is_error

  let both a b =
    match (a, b) with
    | (Ok a', Ok b') ->
        Ok (a', b')
    | (Error a', _) ->
        Error a'
    | (_, Error b') ->
        Error b'

  let flatten = Result.join

  let or_ a b = match a with Ok _ -> a | _ -> b

  let and_ a b = match a with Ok _ -> b | _ -> a

  let get = Result.value

  let ( |? ) t default = get t ~default

  let getUnsafe = Result.get_ok

  let getError t ~default =
    match t with Ok _ -> default | Error error -> error

  let map t ~f = Result.map f t

  let map2 a b ~f =
    match (a, b) with
    | (Ok a, Ok b) ->
        Ok (f a b)
    | (Error a, _) ->
        Error a
    | (_, Error b) ->
        Error b

  let mapError t ~f =
    match t with Error error -> Error (f error) | Ok value -> Ok value

  let values t =
    Base.List.fold_right t ~f:(map2 ~f:(fun a b -> a :: b)) ~init:(Ok []) 

  let toOption r = match r with Ok v -> Some v | Error _ -> None

  let flatMap t ~f = Result.bind t f

  let attempt f =
    match f () with value -> Ok value | exception error -> Error error

  let transpose t =
    match t with
    | Error error ->
        Some (Error error)
    | Ok None ->
        None
    | Ok (Some value) ->
        Some (Ok value)

  let forEach t ~f = match t with Ok a -> f a | _ -> ()

  let equal equalOk equalError a b =
    match (a, b) with
    | (Error a', Error b') ->
        equalError a' b'
    | (Ok a', Ok b') ->
        equalOk a' b'
    | _ ->
        false

  let compare (compareOk : 'ok -> 'ok -> int)
      (compareError : 'error -> 'error -> int) (a : ('ok, 'error) t)
      (b : ('ok, 'error) t) =
    ( match (a, b) with
      | (Error a', Error b') ->
          compareError a' b'
      | (Ok a', Ok b') ->
          compareOk a' b'
      | (Error _, Ok _) ->
          -1
      | (Ok _, Error _) ->
          1
      : int )

  module Infix = struct    
    let ( >>= ) t f = flatMap t ~f

    let ( >>| ) t f = map t ~f
  end
end

module Float = struct
  type t = float

  type radians = t

  let ofInt = Base.Float.of_int

  let ofString string =
    try Some (Base.Float.of_string string) with Invalid_argument _ -> None

  let zero = 0.0

  let one = 1.0

  let nan = Base.Float.nan

  let infinity = Base.Float.infinity

  let negativeInfinity = Base.Float.neg_infinity

  let e = Base.Float.euler

  let pi = Base.Float.pi

  let epsilon = Base.Float.epsilon_float

  let maximumSafeInteger = (2. ** 52.) -. 1.

  let minimumSafeInteger = (-2. ** 52.) -. 1.

  let largestValue = Base.Float.max_finite_value

  let smallestValue = Base.Float.min_positive_normal_value

  let add = ( +. )

  let ( + ) = ( +. )

  let subtract = ( -. )

  let ( - ) = ( -. )

  let multiply = ( *. )

  let ( * ) = ( *. )

  let divide n ~by = n /. by

  let ( / ) = ( /. )

  let power ~base ~exponent = base ** exponent

  let ( ** ) = ( ** )

  let negate = Base.Float.neg

  let ( ~- ) = negate

  let absolute = Base.Float.abs

  let isInteger t = t = Base.Float.round t

  let isSafeInteger t = isInteger t && t <= maximumSafeInteger

  let clamp n ~lower ~upper =
    if upper < lower then
      raise
        (Invalid_argument
           ( "~lower:" ^ Base.Float.to_string lower
           ^ " must be less than or equal to ~upper:"
           ^ Base.Float.to_string upper ))
    else if
      Base.Float.is_nan lower || Base.Float.is_nan upper || Base.Float.is_nan n
    then Base.Float.nan
    else max lower (min upper n)

  let inRange n ~lower ~upper =
    if
      let open Base.Float in
      upper < lower
    then
      raise
        (Invalid_argument
           ( "~lower:" ^ Base.Float.to_string lower
           ^ " must be less than or equal to ~upper:"
           ^ Base.Float.to_string upper ))
    else n >= lower && n < upper

  let squareRoot = sqrt

  let log n ~base =
    let open Base.Float in
    log10 n / log10 base

  let isNaN = Base.Float.is_nan

  let isInfinite = Base.Float.is_inf

  let isFinite n = (not (isInfinite n)) && not (isNaN n)

  let maximum x y = if isNaN x || isNaN y then nan else if y > x then y else x

  let minimum x y = if isNaN x || isNaN y then nan else if y < x then y else x

  let hypotenuse x y = squareRoot ((x * x) + (y * y))

  let degrees n = n * (pi / 180.0)

  let radians = Fun.identity

  let turns n = n * 2. * pi

  let cos = Base.Float.cos

  let acos = Base.Float.acos

  let sin = Base.Float.sin

  let asin = Base.Float.asin

  let tan = Base.Float.tan

  let atan = Base.Float.atan

  let atan2 ~y ~x = Base.Float.atan2 y x

  type direction =
    [ `Zero
    | `AwayFromZero
    | `Up
    | `Down
    | `Closest of [`Zero | `AwayFromZero | `Up | `Down | `ToEven] ]

  let round ?(direction = `Closest `Up) n =
    match direction with
    | (`Up | `Down | `Zero) as dir ->
        Base.Float.round n ~dir
    | `AwayFromZero ->
        if n < 0. then Base.Float.round n ~dir:`Down
        else Base.Float.round n ~dir:`Up
    | `Closest `Zero ->
        if n > 0. then Base.Float.round (n -. 0.5) ~dir:`Up
        else Base.Float.round (n +. 0.5) ~dir:`Down
    | `Closest `AwayFromZero ->
        if n > 0. then Base.Float.round (n +. 0.5) ~dir:`Down
        else Base.Float.round (n -. 0.5) ~dir:`Up
    | `Closest `Down ->
        Base.Float.round (n -. 0.5) ~dir:`Up
    | `Closest `Up ->
        Base.Float.round_nearest n
    | `Closest `ToEven ->
        Base.Float.round_nearest_half_to_even n

  let floor = Base.Float.round_down

  let ceiling = Base.Float.round_up

  let truncate = Base.Float.round_towards_zero

  let ofPolar (r, theta) = (r * cos theta, r * sin theta)

  let toPolar (x, y) = (hypotenuse x y, atan2 ~x ~y)

  let toInt = Base.Float.iround_towards_zero

  let toString = Base.Float.to_string

  let equal = ( = )

  let compare = compare
end

module Int = struct
  type t = int

  let ofString = int_of_string_opt

  let minimumValue = Base.Int.min_value

  let maximumValue = Base.Int.max_value

  let zero = 0

  let one = 1

  let add = ( + )

  let ( + ) = ( + )

  let subtract = ( - )

  let ( - ) = ( - )

  let multiply = ( * )

  let ( * ) = multiply

  let divide n ~by = n / by

  let ( / ) = ( / )

  let ( /. ) = Base.Int.( // )

  let power ~base ~exponent =
    let open Base.Int in
    base ** exponent

  let ( ** ) = Base.Int.( ** )

  let negate = ( ~- )

  let ( ~- ) = ( ~- )

  let remainder n ~by = Stdlib.(n mod by)

  let modulo n ~by =     
    ((if n < 0 then 2 * (abs n) else n) mod by)

  let ( mod ) n by = modulo n ~by

  let maximum = Base.Int.max

  let minimum = Base.Int.min

  let absolute n = Base.Int.abs n

  let isEven n = remainder n ~by:2 = 0

  let isOdd n = remainder n ~by:2 <> 0

  let clamp n ~lower ~upper =
    if upper < lower then
      raise
        (Invalid_argument
           ( "~lower:" ^ Base.Int.to_string lower
           ^ " must be less than or equal to ~upper:"
           ^ Base.Int.to_string upper ))
    else max lower (min upper n)

  let inRange n ~lower ~upper =
    if upper < lower then
      raise
        (Invalid_argument
           ( "~lower:" ^ Base.Int.to_string lower
           ^ " must be less than or equal to ~upper:"
           ^ Base.Int.to_string upper ))
    else n >= lower && n < upper

  let toFloat = Base.Int.to_float

  let toString = Base.Int.to_string

  let equal = ( = )

  let compare = compare

  type identity = Base.Int.comparator_witness

  let comparator = Base.Int.comparator
end

module Integer = struct  
  type t = Z.t
  include Comparator.Make(struct 
    type nonrec t = t
    let compare = Z.compare
  end)

  let ofInt = Z.of_int

  let ofInt64 = Z.of_int64

  let ofFloat float =
    match Z.of_float float with
    | integer ->
        Some integer
    | exception Z.Overflow ->
        None

  let ofString string =
    match Z.of_string string with value -> Some value | exception _ -> None

  let zero = Z.zero

  let one = Z.one

  let isEven t =
    let open Z in
    t mod ~$2 = zero

  let isOdd t =
    let open Z in
    t mod ~$2 <> zero

  let add = Z.add

  let ( + ) = Z.( + )

  let subtract = Z.sub

  let ( - ) = subtract

  let multiply = Z.mul

  let ( * ) = multiply

  let divide = Z.div

  let ( / ) = divide

  let divide n ~by = divide n by

  let negate = Z.neg

  let modulo (n : t) ~(by : t) : t = (Z.rem n by)

  let (mod) (n : t) (by : t) : t = (Z.rem n by)

  let remainder (n : t) ~(by : t) = (Z.rem n by)

  let ( ** ) = Z.( ** )

  let power ?modulo ~(base : t) ~(exponent : int) =
    match modulo with
    | Some modulus ->
        Z.powm base (ofInt exponent) modulus
    | None ->
        Z.pow base exponent

  let maximum a b = if a < b then b else a

  let minimum a b = if a > b then b else a

  let absolute n = if n < zero then negate n else n

  let clamp n ~lower ~upper =
    if upper < lower then
      raise (Invalid_argument "~lower must be less than or equal to ~upper")
    else max lower (min upper n)

  let inRange n ~lower ~upper =
    if upper < lower then
      raise (Invalid_argument "~lower must be less than or equal to ~upper")
    else n >= lower && n < upper

  let toInt t = if t > ofInt Int.maximumValue then None else Some (Z.to_int t)

  let toInt64 t =
    if t > ofInt64 Int64.max_int then None else Some (Z.to_int64 t)

  let toFloat = Z.to_float

  let toString = Z.to_string

  let equal = Z.equal

  let compare = Z.compare
end

module String = struct
  type t = string

  let initialize length ~f =
    Base.List.init length ~f |> Base.String.of_char_list

  let repeat t ~count =
    Base.List.init count ~f:(fun _ -> t) |> Base.String.concat

  let ofArray characters =
    let open Base in
    Array.to_list characters |> String.of_char_list

  let ofList = Base.String.of_char_list

  let length = String.length

  let isEmpty t = length t = 0

  let get = Base.String.get

  let getAt a ~index =
    if index >= 0 && index < length a then Some (Base.String.get a index)
    else None

  let (.?[]) (string: string) (index: int) : char option = 
    getAt string ~index

  let uncons (s : string) =
    ( match s with
      | "" ->
          None
      | s ->
          Some (s.[0], String.sub s 1 (String.length s - 1))
      : (char * string) option )

  let dropLeft (s : string) ~(count : int) =
    (Base.String.drop_prefix s count : string)

  let dropRight (s : string) ~(count : int) =
    (Base.String.drop_suffix s count : string)

  let split t ~(on : string) =
    (Str.split (Str.regexp_string on) t : string list)

  let startsWith t ~prefix = Base.String.is_prefix ~prefix t

  let endsWith t ~suffix = Base.String.is_suffix ~suffix t

  let toLowercase (s : string) = (String.lowercase_ascii s : string)

  let toUppercase (s : string) = (String.uppercase_ascii s : string)

  let uncapitalize (s : string) = (String.uncapitalize_ascii s : string)

  let capitalize (s : string) = (String.capitalize_ascii s : string)

  let isCapitalized (s : string) = (s = String.capitalize_ascii s : bool)

  let includes t ~substring = (Base.String.is_substring t ~substring : bool)

  let reverse = Base.String.rev

  let ofChar = Base.String.of_char

  let slice ?(to_ = 0) str ~from = String.sub str from (to_ - from)

  let insertAt t ~(index : int) ~(value : string) =
    ( let length = length t in
      let startCount = index in
      let endCount = length - index in
      let start = dropRight ~count:endCount t in
      let end_ = dropLeft ~count:startCount t in
      String.concat "" [start; value; end_]
      : string )

  let toArray string = Base.String.to_list string |> Array.of_list

  let toList = Base.String.to_list

  let trim string = Base.String.strip string

  let trimLeft string = Base.String.lstrip string

  let trimRight string = Base.String.rstrip string

  let padLeft string targetLength ~with_ = 
    if (length(string) >= targetLength) then 
      string 
    else (
      let paddingLength = targetLength - length(string) in
      let count = paddingLength / length(with_) in
      let padding = slice (repeat with_ ~count) ~from:0 ~to_:paddingLength in
      padding ^ string
    )

  let padRight string targetLength ~with_ = 
    if (length(string) >= targetLength) then 
      string 
    else (
      let paddingLength = targetLength - length(string) in
      let count = paddingLength / length(with_) in
      let padding = slice (repeat with_ ~count) ~from:0 ~to_:paddingLength in
      string ^ padding
    )

  let forEach = Base.String.iter

  let fold s ~initial ~f = Base.String.fold s ~init:initial ~f

  let equal = Base.String.equal

  let compare = Base.String.compare

  type identity = Base.String.comparator_witness

  let comparator = Base.String.comparator
end

module Set = struct
  type ('a, 'id) t = ('a, 'id) Base.Set.t

  module Of (M : Comparator.S) = struct
    type nonrec 'value t = (M.t, M.identity) t
  end

  let empty comparator = 
    (Base.Set.empty (Comparator.toBaseComparator comparator))    

  let singleton (comparator: ('a, 'identity) Comparator.s) (element: 'a) : ('a, 'identity) t = 
    (Base.Set.of_list (Comparator.toBaseComparator comparator) [element])

  let ofArray (comparator: ('a, 'identity) Comparator.s) (elements: 'a array) : ('a, 'identity) t  = 
    (Base.Set.of_list (Comparator.toBaseComparator comparator) (Array.to_list elements))

  let ofList (comparator: ('a, 'identity) Comparator.s) (elements: 'a list) : ('a, 'identity) t  = 
    (Base.Set.of_list (Comparator.toBaseComparator comparator) elements)

  let length = Base.Set.length

  let isEmpty = Base.Set.is_empty

  let includes = Base.Set.mem

  let (.?{}) (set: ('element, _) t) (element: 'element) : bool = 
    includes set element

  let add = Base.Set.add

  let remove = Base.Set.remove

  let difference = Base.Set.diff

  let intersection = Base.Set.inter

  let union = Base.Set.union

  let filter = Base.Set.filter

  let partition = Base.Set.partition_tf

  let find = Base.Set.find

  let all = Base.Set.for_all

  let any = Base.Set.exists

  let forEach = Base.Set.iter

  let fold s ~initial ~f = Base.Set.fold s ~init:initial ~f

  let toArray = Base.Set.to_array

  let toList = Base.Set.to_list

  module Poly = struct
    type identity = Base.Comparator.Poly.comparator_witness

    type nonrec 'a t = ('a, identity) t

    let empty () = Base.Set.Poly.empty

    let singleton = Base.Set.Poly.singleton

    let ofArray = Base.Set.Poly.of_array

    let ofList = Base.Set.Poly.of_list
  end

  module Int = struct
    type nonrec t = (Base.Int.t, Base.Int.comparator_witness) t

    let empty = Base.Set.empty (module Base.Int)

    let singleton = Base.Set.singleton (module Base.Int)

    let ofArray = Base.Set.of_array (module Base.Int)

    let ofList = Base.Set.of_list (module Base.Int)
  end

  module String = struct
    type nonrec t = (String.t, Base.String.comparator_witness) t

    let empty = Base.Set.empty (module Base.String)

    let singleton = Base.Set.singleton (module Base.String)

    let ofArray = Base.Set.of_array (module Base.String)

    let ofList = Base.Set.of_list (module Base.String)
  end
end

module Map = struct
  type ('key, 'value, 'id) t = ('key, 'value, 'id) Base.Map.t

  module Of (M : Comparator.S) = struct
    type nonrec 'value t = (M.t, 'value, M.identity) t
  end

  let keepLatestOnly = fun _ latest -> latest

  let empty (comparator : ('key, 'identity) Comparator.s) : ('key, 'value, 'identity) t = 
    (Base.Map.empty (Comparator.toBaseComparator comparator))    

  let singleton (comparator: ('key, 'identity) Comparator.s) ~key ~value : ('key, 'value, 'identity) t = 
    Base.Map.of_alist_reduce (Comparator.toBaseComparator comparator) [(key, value)] ~f:keepLatestOnly

  let ofArray (comparator: ('key, 'identity) Comparator.s) (elements: ('key * 'value) array) : ('key, 'value, 'identity) t  = 
    Base.Map.of_alist_reduce (Comparator.toBaseComparator comparator) (Array.to_list elements) ~f:keepLatestOnly

  let ofList (comparator: ('key, 'identity) Comparator.s) (elements: ('key * 'value) list) : ('key, 'value, 'identity) t  = 
    Base.Map.of_alist_reduce (Comparator.toBaseComparator comparator) elements ~f:keepLatestOnly

  let isEmpty = Base.Map.is_empty

  let includes = Base.Map.mem

  let length = Base.Map.length

  let minimum t = Base.Map.min_elt t |> Option.map ~f:Tuple.first

  let maximum t = Base.Map.max_elt t |> Option.map ~f:Tuple.first

  let extent t = Option.both (minimum t) (maximum t)

  let add m ~key ~value = Base.Map.set m ~key ~data:value

  let (.?{}<-) (map: ('key, 'value, 'id) t) (key: 'key) (value: 'value): ('key, 'value, 'id) t = 
    add map ~key ~value

  let remove = Base.Map.remove

  let get = Base.Map.find

  let (.?{}) (map: ('key, 'value, _) t) (key: 'key) : 'value option = 
    get map key  

  let update m ~key ~f = Base.Map.change m key ~f

  let merge m1 m2 ~f =
    Base.Map.merge m1 m2 ~f:(fun ~key desc ->
        match desc with
        | `Left v1 ->
            f key (Some v1) None
        | `Right v2 ->
            f key None (Some v2)
        | `Both (v1, v2) ->
            f key (Some v1) (Some v2))

  let map = Base.Map.map

  let mapI t ~f = Base.Map.mapi t ~f:(fun ~key ~data -> f key data)

  let filter = Base.Map.filter

  let partition m ~f =
    Base.Map.partition_mapi m ~f:(fun ~key ~data ->
        if f ~key ~value:data then `Fst data else `Snd data)

  let find m ~f =
    Base.Map.fold m ~init:None ~f:(fun ~key ~data matching ->
        match matching with
        | Some _ ->
            matching
        | None ->
            if f ~key ~value:data then Some (key, data) else None)

  let any = Base.Map.exists

  let all = Base.Map.for_all

  let forEach = Base.Map.iter

  let forEachI (map : ('key, 'value, _) t) ~(f:key:'key -> value:'value -> unit) : unit = 
    Base.Map.iteri map ~f:(fun ~key ~data -> f ~key ~value:data)

  let fold m ~initial ~f =
    Base.Map.fold m ~init:initial ~f:(fun ~key ~data acc ->
        f acc ~key ~value:data)

  let keys = Base.Map.keys

  let values = Base.Map.data

  let toArray m = Base.Map.to_alist m |> Base.List.to_array

  let toList m = Base.Map.to_alist m

  module Poly = struct
    type identity = Base.Comparator.Poly.comparator_witness

    type nonrec ('k, 'v) t = ('k, 'v, identity) t

    let empty () = Base.Map.Poly.empty

    let singleton ~key ~value = Base.Map.Poly.singleton key value

    let ofList l = Base.Map.Poly.of_alist_reduce l ~f:(fun _ curr -> curr)

    let ofArray a = Base.Array.to_list a |> ofList
  end

  module Int = struct
    type nonrec 'v t = (Int.t, 'v, Int.identity) t

    let empty = Base.Map.empty (module Base.Int)

    let singleton ~key ~value = Base.Map.singleton (module Base.Int) key value

    let ofList l =
      Base.Map.of_alist_reduce (module Base.Int) l ~f:(fun _ curr -> curr)

    let ofArray a = Base.Array.to_list a |> ofList
  end

  module String = struct
    type nonrec 'v t = (String.t, 'v, Base.String.comparator_witness) t

    let empty = Base.Map.empty (module Base.String)

    let singleton ~key ~value =
      Base.Map.singleton (module Base.String) key value

    let ofList l =
      Base.Map.of_alist_reduce (module Base.String) l ~f:(fun _ curr -> curr)

    let ofArray a = Base.Array.to_list a |> ofList
  end
end

module Array = struct
  type 'a t = 'a array

  let singleton (a : 'a) = ([|a|] : 'a array)

  let clone = Base.Array.copy

  let initialize (length : int) ~(f : int -> 'a) =
    if length <= 0 then [||] else Base.Array.init length ~f

  let repeat element ~length = initialize length ~f:(Fun.constant element)

  let range ?(from = 0) (to_ : int) =
    (Base.Array.init (max 0 (to_ - from)) ~f:(fun i -> i + from) : int array)

  let ofList = Base.List.to_array

  let length (a : 'a array) = (Base.Array.length a : int)

  let isEmpty (a : 'a array) = (length a = 0 : bool)

  let first t = if length t < 1 then None else Some t.(0)

  let last t = if length t < 1 then None else Some t.(length t - 1)

  let get = Base.Array.get

  let getAt a ~index =
    if index >= 0 && index < length a then Some (Base.Array.get a index)
    else None

  let (.?()) (array: 'element t) (index: int) : 'element option = 
    getAt array ~index

  let set = Base.Array.set

  let setAt t ~index ~value = set t index value

  let filter = Base.Array.filter

  let sum (type a) t (module M : Container.Sum with type t = a) =
    Base.Array.fold t ~init:M.zero ~f:M.add

  let filterMap = Base.Array.filter_map

  let flatMap = Base.Array.concat_map

  let fold a ~initial ~f = Base.Array.fold a ~init:initial ~f

  let foldRight a ~initial ~f =
    Base.Array.fold_right a ~init:initial ~f:(Fun.flip f)

  let count t ~f =
    fold t ~initial:0 ~f:(fun total element ->
        total + match f element with true -> 1 | false -> 0)

  let swap = Base.Array.swap

  let find = Base.Array.find

  let findIndex = Base.Array.findi

  let map = Base.Array.map

  let mapI = Base.Array.mapi

  let map2 (a : 'a array) (b : 'b array) ~(f : 'a -> 'b -> 'c) =
    ( let minLength = min (length a) (length b) in
      Base.Array.init minLength ~f:(fun i -> f a.(i) b.(i))
      : 'c array )

  let zip = map2 ~f:Tuple.make

  let map3 (arrayA : 'a array) (arrayB : 'b array) (arrayC : 'c array)
      ~(f : 'a -> 'b -> 'c -> 'd) =
    let minLength =
      Base.min (length arrayA) (Base.min (length arrayC) (length arrayB))
    in
    Base.Array.init minLength ~f:(fun i -> f arrayA.(i) arrayB.(i) arrayC.(i))

  let partition = Base.Array.partition_tf

  let splitAt a ~index =
    ( Base.Array.init index ~f:(fun i -> a.(i))
    , Base.Array.init (length a - 1) ~f:(fun i -> a.(index + i)) )

  let splitWhen a ~f =
    match findIndex a ~f:(fun _index element -> f element) with
    | None ->
        (a, [||])
    | Some (index, _) ->
        splitAt a ~index

  let unzip = Base.Array.unzip

  let append (a : 'a array) (a' : 'a array) =
    (Base.Array.append a a' : 'a array)

  let flatten (al : 'a array array) =
    (Base.Array.concat (Base.Array.to_list al) : 'a array)

  let intersperse array ~sep =
    Base.Array.init
      (max 0 ((Array.length array * 2) - 1))
      ~f:(fun i -> if i mod 2 <> 0 then sep else array.(i / 2))

  let any = Base.Array.exists

  let all = Base.Array.for_all

  let includes = Base.Array.mem

  let values t =
    fold t ~initial:[] ~f:(fun results element ->
        match element with None -> results | Some value -> value :: results)
    |> ofList

  let join t ~sep = Stdlib.String.concat sep (Array.to_list t)

  let slice ?to_ array ~from =
    let defaultTo = match to_ with None -> length array | Some i -> i in
    let sliceFrom =
      if from >= 0 then min (length array) from
      else max 0 (min (length array) (length array + from))
    in
    let sliceTo =
      if defaultTo >= 0 then min (length array) defaultTo
      else max 0 (min (length array) (length array + defaultTo))
    in
    if sliceFrom >= sliceTo then [||]
    else
      Base.Array.init (sliceTo - sliceFrom) ~f:(fun i -> array.(i + sliceFrom))

  let sliding ?(step = 1) a ~size =
    let n = Array.length a in
    if size > n then [||]
    else
      initialize
        (1 + ((n - size) / step))
        ~f:(fun i -> initialize size ~f:(fun j -> a.((i * step) + j)))

  let chunksOf t ~size = sliding t ~step:size ~size

  let maximum = Base.Array.max_elt

  let minimum = Base.Array.min_elt

  let extent t ~compare =
    fold t ~initial:None ~f:(fun range element ->
        match range with
        | None ->
            Some (element, element)
        | Some (min, max) ->
            Some
              ( ( match compare element min < 0 with
                | true ->
                    element
                | false ->
                    min )
              , match compare element max > 0 with
                | true ->
                    element
                | false ->
                    max ))

  let sort t = Base.Array.sort t

  let reverse = Base.Array.rev_inplace

  let forEach a ~f = Base.Array.iter a ~f

  let forEachI a ~f = Base.Array.iteri a ~f

  let groupBy t comparator ~f =
    fold t ~initial:(Map.empty comparator) ~f:(fun map element -> (
      let key = f element in
      Map.update map ~key ~f:(function
          | None -> Some [element]
          | Some elements -> Some (element :: elements)
      )
    ))

  let toList (a : 'a array) = (Base.Array.to_list a : 'a list)

  let toIndexedList a =
    Base.Array.fold_right a
      ~init:(length a - 1, [])
      ~f:(fun x (i, acc) -> (i - 1, (i, x) :: acc))
    |> Base.snd

  let equal equal a b =
    if length a <> length b then false
    else if length a = 0 then true
    else (
      let rec loop index =
        if index = length a then true
        else equal a.(index) b.(index) && loop (index + 1)
      in
      loop 0 )

  let compare compare a b =
    match Int.compare (length a) (length b) with
    | 0 ->
        if length a == 0 then 0
        else (
          let rec loop index =
            if index = length a then 0
            else (
              match compare a.(index) b.(index) with
              | 0 ->
                  loop (index + 1)
              | result ->
                  result )
          in
          loop 0 )
    | result ->
        result
end

module List = struct
  type 'a t = 'a list

  let empty = []

  let singleton = Base.List.return

  let repeat element ~times = Base.List.init times ~f:(fun _ -> element)

  let rec range ?(from = 0) to_ =
    if from >= to_ then [] else from :: range ~from:(from + 1) to_

  let initialize = Base.List.init

  let sum (type a) (a : a t) (module M : Container.Sum with type t = a) =
    (Base.List.fold a ~init:M.zero ~f:M.add : a)

  let ofArray = Base.Array.to_list

  let isEmpty (l : 'a list) = (l = [] : bool)

  let head = Base.List.hd

  let tail = Base.List.tl

  let cons list element = element :: list

  let take t ~count = Base.List.take t count

  let takeWhile (l : 'a list) ~(f : 'a -> bool) =
    ( let rec takeWhileHelper acc l' =
        match l' with
        | [] ->
            Base.List.rev acc
        | x :: rest ->
            if f x then takeWhileHelper (x :: acc) rest else Base.List.rev acc
      in
      takeWhileHelper [] l
      : 'a list )

  let drop t ~count = Base.List.drop t count

  let rec dropWhile (l : 'a list) ~(f : 'a -> bool) =
    ( match l with
      | [] ->
          []
      | x :: rest ->
          if f x then dropWhile ~f rest else l
      : 'a list )

  let initial (l : 'a list) =
    ( match Base.List.rev l with
      | [] ->
          None
      | _ :: rest ->
          Some (Base.List.rev rest)
      : 'a list option )

  let rec last (l : 'a list) =
    ( match l with [] -> None | [a] -> Some a | _ :: tail -> last tail
      : 'a option )

  let append (l1 : 'a list) (l2 : 'a list) = (Base.List.append l1 l2 : 'a list)

  let flatten = Base.List.concat

  let map2 = Base.List.map2_exn

  let map3 = Base.List.map3_exn

  let reverse (l : 'a list) = (Base.List.rev l : 'a list)

  let map = Base.List.map

  let mapI = Base.List.mapi

  let flatMap = Base.List.concat_map

  let includes = Base.List.mem

  let find = Base.List.find

  let findIndex = Base.List.findi

  let any = Base.List.exists

  let all = Base.List.for_all

  let getAt (l : 'a list) ~(index : int) = (Base.List.nth l index : 'a option)

  let filterMap = Base.List.filter_map

  let filter t ~f = Base.List.filter t ~f

  let filterI t ~f = Base.List.filteri t ~f

  let partition = Base.List.partition_tf

  let fold t ~initial ~f = Base.List.fold t ~init:initial ~f

  let count = Base.List.count

  let foldRight t ~initial ~f =
    Base.List.fold_right t ~init:initial ~f:(Fun.flip f)

  let splitAt (l : 'a list) ~(index : int) =
    ((take ~count:index l, drop ~count:index l) : 'a list * 'a list)

  let splitWhen (l : 'a list) ~(f : 'a -> bool) =
    ( match findIndex ~f:(fun _ element -> f element) l with
      | Some (index, _) ->
          splitAt ~index l
      | None ->
          (l, [])
      : 'a list * 'a list )

  let updateAt (l : 'a list) ~(index : int) ~(f : 'a -> 'a) =
    ( if index < 0 then l
      else (
        let (front, back) = splitAt ~index l in
        match back with [] -> l | x :: rest -> append front (f x :: rest) )
      : 'a list )

  let length (l : 'a list) = (List.length l : int)

  let removeAt (l : 'a list) ~(index : int) =
    ( if index < 0 then l
      else (
        let (front, back) = splitAt ~index l in
        match tail back with None -> l | Some t -> append front t )
      : 'a list )

  let minimum = Base.List.min_elt

  let maximum = Base.List.max_elt

  let extent t ~compare =
    fold t ~initial:None ~f:(fun current element ->
        match current with
        | None ->
            Some (element, element)
        | Some (min, max) ->
            Some
              ( ( match compare element min < 0 with
                | true ->
                    element
                | false ->
                    min )
              , match compare element max > 0 with
                | true ->
                    element
                | false ->
                    max ))

  let insertAt (t : 'a list) ~(index : int) ~(value : 'a) =
    ( let (front, back) = splitAt t ~index in
      append front (value :: back)
      : 'a list )

  let zip listA listB = 
    let rec loop result xs ys = 
      match (xs, ys) with
      | [], _ -> result
      | _, [] -> result
      | x :: xs, y :: ys -> loop ((x, y) :: result) xs ys
    in
    loop [] listA listB

  let unzip = Base.List.unzip

  let sliding ?(step = 1) (t : 'a t) ~(size : int) =
    ( let rec takeAllOrEmpty t n (current, count) =
        if count = n then reverse current
        else (
          match t with
          | [] ->
              []
          | x :: xs ->
              takeAllOrEmpty xs n (x :: current, count + 1) )
      in
      let rec loop t =
        if isEmpty t then []
        else (
          let sample = takeAllOrEmpty t size ([], 0) in
          if isEmpty sample then [] else sample :: loop (Base.List.drop t step)
          )
      in
      loop t
      : 'a t t )

  let chunksOf t ~size = sliding t ~step:size ~size

  let intersperse (l : 'a list) ~sep =
    ( match l with
      | [] ->
          []
      | [x] ->
          [x]
      | x :: rest ->
          x :: foldRight rest ~initial:[] ~f:(fun acc x -> sep :: x :: acc)
      : 'a list )

  let forEach l ~f = Base.List.iter l ~f

  let forEachI = Base.List.iteri

  let toArray = Base.List.to_array

  let groupWhile t ~f = Base.List.group t ~break:f

  let sort = Base.List.sort

  let join t ~sep = Stdlib.String.concat sep t

  let groupBy t comparator ~f =
    fold t ~initial:(Map.empty comparator) ~f:(fun map element -> (
      let key = f element in
      Map.update map ~key ~f:(function
          | None -> Some [element]
          | Some elements -> Some (element :: elements)
      )
    ))

  let rec equal equalElement a b =
    match (a, b) with
    | ([], []) ->
        true
    | (x :: xs, y :: ys) ->
        equalElement x y && equal equalElement xs ys
    | _ ->
        false

  let rec compare compareElement a b =
    match (a, b) with
    | ([], []) ->
        0
    | ([], _) ->
        -1
    | (_, []) ->
        1
    | (x :: xs, y :: ys) -> (
      match compareElement x y with
      | 0 ->
          compare compareElement xs ys
      | result ->
          result )
end

let () = ()