You can search for identifiers within the package.
in-package search v0.2.0
reason-standard
Standard.Container
Interfaces for use with container types like Array or List
Array
List
module type Sum = sig ... end
Modules which conform to this signature can be used with functions like Array.sum or List.sum
Array.sum
List.sum